@@ -5,6 +5,7 @@ use Carp;
 use XML::LibXML;
 use SOAP::Lite;
 use SOAP::Data::Builder;
+use File::Slurp;
 
 use vars qw($VERSION);
 
@@ -14,89 +15,14 @@ my @methods = qw(results results_xml uri xmlns proxy soapversion timeout error s
 
 __PACKAGE__->mk_accessors(@methods);
 
-$VERSION = 1.1;
-
-=head1 NAME
-
-SOAP::Lite::Simple - Simple frame work for talking with web services
-
-=head1 DESCRIPTION
-
-This package is the base class for talking with web services,
-there are specific modules to use depending on the type
-of service you are calling, e.g. <SOAP::Lite::Simple::DotNet> or
-<SOAP::Lite::Simple::Real>
-
-This package helps in talking with web services, it just needs
-a bit of XML thrown at it and you get some XML back.
-It's designed to be REALLY simple to use, it doesn't try to 
-be cleaver in any way (patches for 'cleaverness' welcome).
-
-=head1 SYNOPSIS
-
-  See SOAP::Lite::Simple::DotNet for usage example.
-
-  If you are creating a child class you just need to
-  impliment the actual _call();
-
-  package SOAP::Lite::Simple::<PACKAGE NAME>;
-
-  use base qw(SOAP::Lite::Simple);
-
-  sub _call {
-	my ($self,$method) = @_;
-
-	# Impliment it! - below is the code from Simple::DotNet
-
-	# This code is the .NET specific way of calling SOAP,
-	# it might work for other stuff as well        
-        my $soap_action = sub {return $self->uri() . '/' . $method};
-        
-        my $caller = $self->{soap}
-                        ->uri($self->uri())
-                        ->proxy($self->proxy(), timeout => $self->timeout())
-                        ->on_action( $soap_action );
-        
-        $caller->soapversion($self->soapversion());
-
-        # Create a SOAP::Data node for the method name
-        my $method_name = SOAP::Data->name($method)->attr({'xmlns' => $self->xmlns()});
-  
-        # Execute the SOAP Request and get the resulting XML
-        my $res = $caller->call( $method_name => $self->{sdb}->to_soap_data());
-
-        return $res;
-  }
-
-  1;
-
-=head1 methods
-
-=head2 new()
-
-  my $soap_simple->SOAP::Lite::Simple::DotNet->new({
-    uri 	=> 'http://www.yourdomain.com/services',
-    proxy 	=> 'http://www.yourproxy.com/services/services.asmx',
-    xmlns 	=> 'http://www.yourdomain.com/services',
-    soapversion => '1.1', # defaults to 1.1
-    timeout	=> '30', # detauls to 30 seconds
-    strip_default_xmlns => 1, # defaults to 1
-  });
-
-This constructor requires uri, proxy and xmlns to be
-supplied, otherwise it will croak.
-
-strip_default_xmlns is used to remove xmlns="http://.../"
-from returned XML, it will NOT alter xmlns:FOO="http//.../"
-set to '0' if you do not wish for this to happen.
-
-=cut
+$VERSION = 1.2;
 
 # Get an XML Parser
 my $parser = XML::LibXML->new();
 $parser->validation(0);
 $parser->expand_entities(0);
 
+# which methods should be set on object constructor
 my @config_methods = qw(uri xmlns proxy soapversion strip_default_xmlns);
 
 sub new {
@@ -128,52 +54,37 @@ sub new {
 
 };
 
-=head2 fetch()
-
-  # Generate the required XML, this is the bit after the Method XML element
-  # in the services.asmx descriptor for this method (see Soap::Lite::Simple::DotNet SYNOPSIS).
-  my $user_id = '900109';
-  my $xml = "<userId _value_type='long'>$user_id</userId>";
-
-  if(my $xml_result = $soap_simple->fetch({ method => 'GetActivity', xml => $xml }) {
-	# You got some XML back
-	my $parser = XML::LibXML->new();
-	my $doc = $parser->parse_string($xml_result);
-
-	# now validate the XML is what you were expecting.
-
-  } else {
-	# There was some sort of error
-	print $soap_simple->error() . "\n";
-  }
-
-This method actually calls the web service, it takes a method name
-and an xml string. If there is a problem with either the XML or
-the SOAP transport (e.g. web server error/could not connect etc)
-undef will be returned and the error() will be set.
-
-If all is successful the the XML string will be parsed back.
-This still has all the SOAP wrapper stuff on it, so you'll
-want to strip that out. 
-
-We check for Fault/faultstring in the returned XML,
-anything else you'll need to check for yourself.
-
-=cut
-
 sub fetch {
 	my ($self,$conf) = @_;
 
-	# Check we have at least an empty string	
-	if(!defined $conf->{xml}) {
-		$self->error('You must supply at least an empty string for the xml');
-		return undef;
-	}
+	# Reset the error so that the object ca be reused
+	$self->error(undef);
+
+	# Got to have a method!
 	if(!defined $conf->{method} or $conf->{method} eq '') {
 		$self->error('You must supply a method name');
 		return undef;
 	}
 
+	# Got to get xml from somewhere!
+	if(!defined $conf->{xml} && !defined $conf->{filename}) {
+		$self->error("You must supply either the 'xml' or the 'filename' to use");
+		return undef;
+	}
+
+	# Check the filename if supplied
+	if(defined $conf->{filename}) {
+		# Got a filename, see if it is readable
+		unless( -r $conf->{filename}) {
+			$self->error("Unable to read: " . $conf->{filename});
+			return undef;
+		} else {
+			# Ok, read it in
+			my $file_xml = read_file( $conf->{filename} );
+			$conf->{xml} = $file_xml;
+		}
+	}
+
 	# add some wrapping paper so XML::LibXML likes it with no top level
 	$conf->{xml} = '<soap_lite_wrapper>' . $conf->{xml} . '</soap_lite_wrapper>';
 	my $xml;
@@ -199,15 +110,15 @@ sub fetch {
 	## Execute the call and get the result back
 	################
 
-	# execute the call in the relevant style
+	# execute the call in the relevant style done by the child object
 	my $res = $self->_call($conf->{method});
 
 	if (!defined $res or $res =~ /^\d/) {
-                # Got a web error - well, if it was XML it wouldn't start with a digit!
+                # Got a web error - if it was XML it wouldn't start with a digit!
                 $self->error($res);
                 return undef;
         } else {
-		# Strip out crap default name space stuff as it makes it hard
+		# Strip out default name space stuff as it makes it hard
 		# to parse and there's no reason for it I can see!
 		$res =~ s/xmlns=".+?"//g if $self->strip_default_xmlns();	
 
@@ -241,35 +152,13 @@ sub fetch {
 			# All looking good
 			$self->results_xml($res_xml);
 			$self->results($res);
-			return $res;
+
+			# I tried just return; but it didn't like it!
+			return 1;
 		}
 	}
 }
 
-=head2 error()
-
-  $self->error();
-
-If fetch returns undef then check this method, it will either be that the XML
-you supplied was not correctly formatted and XML::LibXML could not parse it, there was
-a transport error with the web service or either soap:Fault and soapenv:Fault error
-messages were returned in the XML.
-
-=head2 results();
-
-  my $results = $soap_simple->results();
-
-Can be called after fetch() to get the raw XML, if fetch was sucessful.
-
-=head2 results_xml();
-
-  my $results_as_xml = $soap_simple->results_xml();
-
-Can be called after fetch() to get the XML::LibXML Document element of the returned
-xml, as long as fetch was sucessful.
-
-=cut
-
 ### Private methods
 
 # Convert the XML to SOAP::Data::Builder
@@ -330,6 +219,108 @@ sub _process_node {
 	}
 }
 
+1;
+
+__END__
+
+=head1 NAME
+
+SOAP::Lite::Simple - Simple frame work for talking with web services
+
+=head1 DESCRIPTION
+
+This package is the base class for talking with web services,
+there are specific modules to use depending on the type
+of service you are calling, e.g. C<SOAP::Lite::Simple::DotNet> or
+C<SOAP::Lite::Simple::Real>
+
+This package helps in talking with web services, it just needs
+a bit of XML thrown at it and you get some XML back.
+It's designed to be REALLY simple to use.
+
+=head1 SYNOPSIS
+
+  See SOAP::Lite::Simple::DotNet for usage example.
+
+  If you are creating a child class you just need to
+  impliment the actual _call() - see pod below.
+
+=head1 methods
+
+=head2 new()
+
+  my $soap_simple->SOAP::Lite::Simple::DotNet->new({
+    uri 	=> 'http://www.yourdomain.com/services',
+    proxy 	=> 'http://www.yourproxy.com/services/services.asmx',
+    xmlns 	=> 'http://www.yourdomain.com/services',
+    soapversion => '1.1',     # defaults to 1.1
+    timeout	=> '30',      # detauls to 30 seconds
+    strip_default_xmlns => 1, # defaults to 1
+  });
+
+This constructor requires uri, proxy and xmlns to be
+supplied, otherwise it will croak.
+
+strip_default_xmlns is used to remove xmlns="http://.../"
+from returned XML, it will NOT alter xmlns:FOO="http//.../"
+set to '0' if you do not wish for this to happen.
+
+=head2 fetch()
+
+  # Generate the required XML, this is the bit after the Method XML element
+  # in the services.asmx descriptor for this method (see Soap::Lite::Simple::DotNet SYNOPSIS).
+  my $user_id = '900109';
+  my $xml = "<userId _value_type='long'>$user_id</userId>";
+
+  if($soap_simple->fetch({ method => 'GetActivity', xml => $xml }) {
+      # Get result as a string
+      my $xml_string = $soap_simple->result();
+
+      # Get result as a XML::LibXML object
+      my $xml_libxml_object = $soap_simple->result_xml();
+
+  } else {
+      # There was some sort of error
+      print $soap_simple->error() . "\n";
+  }
+
+This method actually calls the web service, it takes a method name
+and an xml string. If there is a problem with either the XML or
+the SOAP transport (e.g. web server error/could not connect etc)
+undef will be returned and the error() will be set.
+
+Each node in the XML supplied (either by string or from a filename)
+needs to have _value_type defined or the submitted format will
+default to 'string'.
+
+You can supply 'filename' rather than 'xml' and it will read in from
+the file.
+
+We check for Fault/faultstring in the returned XML,
+anything else you'll need to check for yourself.
+
+=head2 error()
+
+  $soap_simple->error();
+
+If fetch returns undef then check this method, it will either be that the filename you
+supplied couldn't be read, the XML you supplied was not correctly formatted (XML::LibXML 
+could not parse it), there was a transport error with the web service or Fault/faultstring
+was found in the XML returned.
+
+=head2 results();
+
+  my $results = $soap_simple->results();
+
+Can be called after fetch() to get the raw XML, if fetch was sucessful.
+
+=head2 results_xml();
+
+  my $results_as_xml = $soap_simple->results_xml();
+
+Can be called after fetch() to get the XML::LibXML Document element of the returned
+xml, as long as fetch was sucessful.
+
 =head1 HOW TO DEBUG
 
 At the top of your script, before 'use SOAP::Lite::Simple::<TYPE>' add:
@@ -347,6 +338,41 @@ or typo in xmlns line.
 If the type of module (e.g. SOAP::Lite::Simple::DotNet) doesn't work, switch
 to one of the other ones and see if that helps. 
 
+=head2 _call()
+
+  This should be implimented by the child class
+
+  package SOAP::Lite::Simple::<PACKAGE NAME>;
+
+  use base qw(SOAP::Lite::Simple);
+
+  sub _call {
+	my ($self,$method) = @_;
+
+	# Impliment it! - below is the code from Simple::DotNet
+
+	# This code is the .NET specific way of calling SOAP,
+	# it might work for other stuff as well        
+        my $soap_action = sub {return $self->uri() . '/' . $method};
+        
+        my $caller = $self->{soap}
+                        ->uri($self->uri())
+                        ->proxy($self->proxy(), timeout => $self->timeout())
+                        ->on_action( $soap_action );
+        
+        $caller->soapversion($self->soapversion());
+
+        # Create a SOAP::Data node for the method name
+        my $method_name = SOAP::Data->name($method)->attr({'xmlns' => $self->xmlns()});
+  
+        # Execute the SOAP Request and get the resulting XML
+        my $res = $caller->call( $method_name => $self->{sdb}->to_soap_data());
+
+        return $res;
+  }
+
+  1;
+
 =head1 SEE ALSO
 
 <SOAP::Lite::Simple::DotNet> <SOAP::Lite::Simple::Real>
@@ -369,5 +395,3 @@ to Aaron for his help with understanding SOAP a bit more and
 the London.pm list for ideas.
 
 =cut
-
-1;
